uniform sampler2D Tex0, Tex1, Tex2;
uniform vec4 lightPos, lightColor;
uniform float lightRadius, lightIntensity; 
uniform bool Debug;

void main(void)
{
	vec3 n, lightDir, l, r;
	float NdotL, RdotE;
	float dist, att;

	vec4 colorMap = texture2DProj(Tex0, gl_TexCoord[0]);
	vec3 normalMap = texture2DProj(Tex1, gl_TexCoord[0]).xyz;
	vec3 ecPos = texture2DProj(Tex2, gl_TexCoord[0]).xyz;
	n = normalize(normalMap * 2.0 - 1.0);

	lightDir = lightPos.xyz - ecPos;
	dist = length(lightDir);
	l = normalize(lightDir);
	if (Debug == false)
		if (dist > lightRadius)
			discard;
	
	gl_FragColor = vec4(0.0);
	att = clamp(1.0 - dist/lightRadius, 0.0, 1.0) * lightIntensity;

	NdotL = max(dot(n, l), 0.0);

	if (NdotL > 0.0) {
		gl_FragColor += att * (lightColor * NdotL);
		r = reflect(-l, n);
		RdotE = max(dot(r, normalize(-ecPos)), 0.0);
		gl_FragColor += vec4(att * lightColor * pow(RdotE, 16.0));
	}

	gl_FragColor *= colorMap;

	if (Debug) {	
		gl_FragColor = vec4(att);
		if (dist > lightRadius - 0.02 && dist < lightRadius + 0.02)
			gl_FragColor = vec4(1.0, 0.0, 0.0, 0.0);
	}
}
